unit CALIBu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, Spin;
  
type
  TMainFrm = class(TForm)
    CardName: TLabel;
    AddressList: TComboBox;
    Label1: TLabel;
    CalibLabel: TLabel;
    Memo1: TMemo;
    RangeLabel: TLabel;
    RangeLabel3: TLabel;
    CalibLabel2: TLabel;
    timer: TTimer;
    Panel3: TPanel;
    RangesImage: TImage;
    ExitButton: TBitBtn;
    ZeroButton: TButton;
    FSVButton: TButton;
    DoneButton: TButton;
    RangesButton: TButton;
    RangesBox: TComboBox;
    RangeLabel2: TLabel;
    procedure ExitButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AddressListChange(Sender: TObject);
    procedure ZeroButtonClick(Sender: TObject);
    procedure BoxChange(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure DoneButtonClick(Sender: TObject);
    procedure FSVButtonClick(Sender: TObject);
    procedure RangesBoxChange(Sender: TObject);
    procedure RangesButtonClick(Sender: TObject);
    procedure timerTimer(Sender: TObject);

  private
    procedure Initialize;
    procedure Finalize;
  public
  { Public declarations }
  end;

var
  MainFrm: TMainFrm;

implementation

uses
  AIOWDM;

{$R *.DFM}

type
  TPCI_COMMON_CONFIG = record
     VendorID : word;
     DeviceID : word;
     Command : word;
     Status : word;
     RevisionID : byte;
     ProgIf : byte;
     SubClass : byte;
     BaseClass : byte;
     CacheLineSize : byte;
     LatencyTimer : byte;
     HeaderType : byte;
     BIST : byte;
     BaseAddresses : Array[0..5] of longint;
     Reserved1 : Array[0..1] of longint;
     RomBaseAddress : longint;
     Reserved2 : Array[0..1] of longint;
     InterruptLine : byte;
     InterruptPin : byte;
     MinimumGrant : byte;
     MaximumLatency : byte;
  end;

  TState = (stRange, stWait, stZero, stFSV);

var
  Address : WORD;
  Address2 : Word;
  Found: Boolean;
  channel: Integer;
  State: TState;
  pic:array[0..15] of TImage;
  chbox:array[0..15] of TComboBox;  //range
  chbutton:array[0..15] of TButton; //set button
  chlabel:array[0..15] of TLabel;   //range label
  chnlabel:array[0..15] of TLabel;  //channel label
  chedit:array[0..15] of TSpinEdit; //spin edit box

procedure WriteCal(ch, cal:word);
var oldtime:longword;
begin
  oldtime:=gettickcount;
  repeat until (gettickcount-oldtime)>10;  //delay 10 ms
  if State = stFSV then
    outportb((address2+(ch*2)+(CHBox[ch].itemindex)*32)+1,4095-cal)
  else
    outportb((address2+(ch*2)+(CHBox[ch].itemindex)*32),cal)
  ;
end;

function ReadCal(ch:word):word;
var oldtime:longword;
begin
  oldtime:=gettickcount;
  repeat until (gettickcount-oldtime)>10;  //delay 10 ms
  if State = stFSV then
    ReadCal:=4095-inportb((address2+(ch*2)+(CHBox[ch].itemindex)*32)+1)
  else
    ReadCal:=inportb((address2+(ch*2)+(CHBox[ch].itemindex)*32))
  ;
end;

procedure SwitchDisplay(Image: TImage; n: word);
begin
 with Image.Canvas do begin
   brush.style := bsSolid;
   brush.color :=clWhite;
   pen.style := psSolid;
   pen.color := clBlack;
   Rectangle(0,0,31,30);  //outer rectangle
   Rectangle(2,2,10,28);  //left rect.
   Rectangle(12,2,20,28); //middle
   Rectangle(22,2,29,28); //right
   brush.style := bsSolid;
   brush.color := $0000cf; //blue:green:read, some red,orangish color
   //draw switches
   FillRect(Rect(3,3+11*(ORD((n AND 4)<>4)),9,14+14*(ORD((n AND 4)<>4))));
   FillRect(Rect(13,3+11*(ORD((n AND 2)<>2)),19,14+14*(ORD((n AND 2)<>2))));
   FillRect(Rect(23,3+11*(ORD((n AND 1)<>1)),28,14+14*(ORD((n AND 1)<>1))));
  end;
end;

procedure TMainFrm.FormCreate(Sender: TObject);
var
  num, i: Integer;
  DeviceID, Base, CalBase: LongWord;

begin
  State := stRange;
  channel := -1;
  found := false;
  num := GetNumCards;
  for i := 0 to num-1 do begin
    QueryCardInfo(I, @DeviceID, @Base, nil, nil);
    QueryBARBase(I, 3, @CalBase);
    case (DeviceID) of
      $6CB0:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':16');
        found := true;
      end;
      $6CA8:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':8');
        found := true;
      end;
      $6CA0:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':6');
        found := true;
      end;
      $6C98:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':4');
        found := true;
      end;
      $6C90:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':2');
        found := true;
      end;
      $6CB1:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':16V');
        found := true;
      end;
      $6CA9:begin
        AddressList.Items.Add('BASE: ' + IntToHex(Base, 4) + ':' + IntToHex(CalBase, 4) + ':8V');
        found := true;
      end;
    end;{case}
  end;{for}
  if not found then
  begin
  //  CardName.Caption:='No Card Found In Registry!';
    Memo1.Lines.Clear;
    Memo1.Lines.Append('No Card Found In Registry!');
    Memo1.Lines.Append('This may mean the card is not installed, or that NTioPCI.SYS is not installed, or the card you have installed is not a PCI-DAC card.');
    Memo1.Lines.Append('If you are uncertain if NTioPCI is installed, please run PCIFind.EXE.');
    AddressList.Items.Add('No Card Found');
  end;
  RangesButton.Enabled := Found;
  AddressList.ItemIndex:=0;
  Rangesbox.ItemIndex:=0;

  if Found then for i:=0 to 15 do begin
    pic[i]:=TImage.Create(self); //create switch picture
    pic[i].left:=(i mod 4)*152+130;
    pic[i].top:=(i div 4)*40+142;
    pic[i].width:=32;
    pic[i].height:=30;
    pic[i].parent:=MainFrm;
    SwitchDisplay(pic[i],0);   //call draw function

    ChnLabel[i] := TLabel.Create(Self);
    ChnLabel[i].Left := (i mod 4)*152+19;
    ChnLabel[i].Top := (i div 4)*40+134;
    ChnLabel[i].Width := 25;
    ChnLabel[i].Height := 13;
    ChnLabel[i].Caption := 'Ch. ' + IntToStr(i);
    ChnLabel[i].Parent := MainFrm;

    ChBox[i]:=TComboBox.Create(self);
    ChBox[i].left:=(i mod 4)*152+70;
    ChBox[i].top:=(i div 4)*40+153;
    ChBox[i].width:=51;
    ChBox[i].height:=21;
    ChBox[i].parent:=MainFrm;
    ChBox[i].Font.Size:=6;
    ChBox[i].Items.Add('0-5V');
    ChBox[i].Items.Add('0-2.5V');
    ChBox[i].Items.Add('0-10V');
    ChBox[i].Items.Add('+/-5V');
    ChBox[i].Items.Add('+/-2.5V');
    ChBox[i].Items.Add('+/-10V');
    ChBox[i].Items.Add('4-20mA');
    ChBox[i].ItemIndex:=0;
    ChBox[i].OnChange:=BoxChange; //assign event handler
    ChBox[i].tag:=i; //set tag so event handler knows how to respond to event

    chedit[i]:=TSpinEdit.Create(self);
    chEdit[i].left:=(i mod 4)*152+16;
    chedit[i].top:=(i div 4)*40+152;
    chedit[i].width:=52;
    chedit[i].height:=22;
    chedit[i].parent:=MainFrm;
    chedit[i].tag:=i;
    chedit[i].minvalue:=0;
    chedit[i].maxvalue:=255;
    chedit[i].value:=ReadCal(i);

    ChButton[i]:=TButton.Create(self);
    ChButton[i].Caption:='Set';
    ChButton[i].left:=(i mod 4)*152+16;
    ChButton[i].top:=(i div 4)*40+148;
    ChButton[i].width:=52;
    ChButton[i].height:=25;
    ChButton[i].parent:=MainFrm;
    ChButton[i].OnClick:=ButtonClick;
    ChButton[i].tag:=i;

    ChLabel[i]:=TLabel.Create(self);
    ChLabel[i].Caption:='';
    ChLabel[i].left:=(i mod 4)*152+80;
    ChLabel[i].top:=(i div 4)*40+153;
    ChLabel[i].parent:=MainFrm;
    ChLabel[i].hide;      //don't show range labels until after buttonclick
  end;
  AddressList.OnChange(AddressList);
end;

// exit button
procedure TMainFrm.ExitButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TMainFrm.Initialize;
var
  i: Integer;
  IsVis: Boolean;

begin
  if State = stRange then begin
    //Methinks it might have been better to have created an array of panels, too...
    for i := 0 to 15 do begin
      IsVis := i <= channel;
      pic[i].Visible := IsVis;
      ChnLabel[i].Visible := IsVis;
      ChBox[i].Visible := IsVis;
      ChEdit[i].Visible := False;
      ChButton[i].Visible := IsVis;
      ChLabel[i].Visible := False;
    end;

    ZeroButton.enabled := False;
    FSVButton.enabled := False;
    DoneButton.Enabled := False;
    RangesButton.Show;
    RangesBox.Show;
  end
  else begin
    if State = stWait then
      for i := 0 to channel do
        ChEdit[i].Visible := False
    else
      for i := 0 to channel do
        ChEdit[i].Visible := True
    ;

    zerobutton.enabled:=true;
    fsvbutton.enabled:=true;
    DoneButton.Enabled := True;
    RangesButton.Hide;
    RangesBox.Hide;
  end;

  for i := 0 to channel do begin //load values from eeprom
    if State = stRange then //This is here because the above is for *all* channels
      ChBox[i].ItemIndex := inportb(address2+240+i)
    ;
    if State = stFSV then begin
      chedit[i].minvalue:=4095-255;
      chedit[i].maxvalue:=4095;
    end else begin
      chedit[i].minvalue:=0;
      chedit[i].maxvalue:=255;
    end;
    chedit[i].value:=readcal(i);
  end;
end;

procedure TMainFrm.Finalize;
var
  I: Integer;

begin //Finalize actually transmits the data to the card
  if (State = stZero) or (State = stFSV) then
    for i:=0 to channel do
      writecal(i, chedit[i].value)
  ;
end;

procedure TMainFrm.AddressListChange(Sender: TObject);
const
  FirstTime: Boolean = True;

var
  ValBuf: String;

begin
  if Found then begin
    if FirstTime then
      FirstTime := False
    else begin
      Finalize;
      Memo1.Lines.Append('New Card Selected: '+AddressList.Items[AddressList.ItemIndex]);
    end;
    Address:=StrToInt('$'+copy(AddressList.Items[AddressList.ItemIndex],7,4));
    Address2:=StrToInt('$'+copy(AddressList.Items[AddressList.ItemIndex],12,4));
    CardName.Caption:='PCI-DA12-'+copy(AddressList.Items[AddressList.ItemIndex],17,3)+' Calibration';
    ValBuf := Copy(AddressList.Items[AddressList.ItemIndex], 17, 2);
    if ValBuf[Length(ValBuf)] = 'V' then
      SetLength(ValBuf, Length(ValBuf)-1)
    ;
    channel := StrToInt(ValBuf)-1;

    inportb(address+15);
    //inportb(address+2);
    inportb(address+10);
    
    State := stRange;
    Initialize;

    MainFrm.ClientHeight := 273 + (channel div 4) * 40;
  end else begin
   Application.MessageBox('This program requires that the PCI-DA12-X be installed.  It was not found.',
                          'Error: Card Not Found',MB_OK);
  end;//Found
end;


procedure TMainFrm.ZeroButtonClick(Sender: TObject);
begin
  timer.enabled:=false;
  ZeroButton.Enabled:=False;

  Finalize;
  CalibLabel.Caption:='Check each channel for zero volts/amps. ';
  CalibLabel2.Caption:='Adjust each channel''s output until voltage is reached.';
  CalibLabel.Show;
  CalibLabel2.Show;
  RangeLabel.Hide;
  RangeLabel3.Hide;
  State := stZero;
  Initialize;

  timer.enabled:=True;
  ZeroButton.Enabled:=True;
end;

procedure TMainFrm.FSVButtonClick(Sender: TObject);
begin
  timer.enabled:=false;
  FSVButton.Enabled:=False;

  Finalize;
  CalibLabel.Caption:='Check each channel for full scale volts/amps.';
  CalibLabel2.Caption:='Adjust each channel''s output until voltage is reached.';
  CalibLabel.Show;
  CalibLabel2.Show;
  RangeLabel.Hide;
  RangeLabel3.Hide;
  State := stFSV;
  Initialize;

  timer.enabled:=true;
  FSVButton.Enabled:=True;
end;


procedure TMainFrm.DoneButtonClick(Sender: TObject);
begin
  timer.enabled:=false;
  DoneButton.Enabled := False;

  Finalize;
  CalibLabel.Hide;
  CalibLabel2.Hide;
  RangeLabel.Show;
  RangeLabel3.Show;
  State := stRange;
  Initialize;

  //ExitButton.Click;
end;

procedure TMainFrm.BoxChange(Sender: TObject);
var
  i:word;
begin
  i:=(sender as TComboBox).ItemIndex;
  SwitchDisplay(pic[(sender as TComboBox).tag],i + 1*(ORD(i>2)));
//  pic[(sender as TComboBox).Tag].Show;
end;

procedure TMainFrm.ButtonClick(Sender: TObject);//set button clicked.

var
  item,i: word;
begin
  //item:=item:=2;
  item:=(sender as TButton).tag;
  ChButton[item].Hide;
  ChLabel[item].Show;
  ChLabel[item].Caption:=ChBox[item].Text;
  ChBox[item].Hide;

  outportb(address2+240+item,chbox[item].ItemIndex);//write cal range
  chedit[item].value:=readcal(item);

  for i:=0 to channel do
    if ChButton[i].Visible then Exit
  ; //Continue only if there are no visible ChButtons
  Finalize;
  State := stWait;
  Initialize;
end;

procedure TMainFrm.RangesBoxChange(Sender: TObject);
begin
  SwitchDisplay(RangesImage,rangesbox.itemindex+1*ord(rangesbox.itemindex>2));
end;

procedure TMainFrm.RangesButtonClick(Sender: TObject);
var
  i:word;
begin
  for i:=0 to channel do begin
    chbox[i].itemindex:=rangesbox.itemindex;
    SwitchDisplay(pic[i],rangesbox.itemindex+1*ord(rangesbox.itemindex>2));
    chbutton[i].Click;
  end;
end;
procedure TMainFrm.timerTimer(Sender: TObject);
var
  i: Integer;

begin
  for i := 0 to channel do
    outport(address+(i*2),chedit[i].Value)
  ;
end;

END.
